/******************************************************
	We'll start by defining a generic gate switch.

	It has 2 states: "open" and "closed"

	Calling flip_switch() toggles this and emits the
	"flipped" signal.

	Individual switches will connect uniquely to
	other objects to receive the event. For now,
	we're just defining the base switch!
******************************************************/

obj/gate_switch
	icon = 'gate_switch.dmi'
	icon_state = "closed"
	density = 1

	var/signal_mgr/signals = new

	verb/flip_switch()
		set src in oview(1)

		if(icon_state == "closed")
			icon_state = "open"
		else
			icon_state = "closed"

		signals.emit("flipped", icon_state)	// emit the signal

/******************************************************
	Next we define the gate to receive the signal.

	The gate object will contain a proc to receive
	the "flip switch" signal emitted by the
	gate_switch. The receiving proc is called a
	"slot."
******************************************************/

obj/gate
	icon = 'gate.dmi'
	icon_state = "closed"
	density = 1
	layer = FLY_LAYER

	proc/change_state(state)
		if(state == "open")
			icon_state = "open"
			density = 0
		else
			icon_state = "closed"
			density = 1

/******************************************************
	Next, we'll make start demonstrating something!

	The demo will contain two switches. The first
	will connect to one gate. The second will connect
	to two gates at once! Connecting signals to slots
	is easy! :D
******************************************************/

obj/gate_switch

	switch_A
		New(loc)
			..(loc)
			spawn(0)
				var/m_gate = locate("gate1_tag")
				if(m_gate)
					signals.connect("flipped", \
						m_gate, /obj/gate/proc/change_state)

		flip_switch()	// Let's make the first switch break!
			..()
			var/m_gate = locate("gate1_tag")
			if(m_gate && signals.disconnect("flipped", m_gate, /obj/gate/proc/change_state))
				world << "[name] breaks!"

	switch_B
		New(loc)
			..(loc)

			spawn(0)
				var/m_gate = locate("gate2_tag")
				if(m_gate)
					signals.connect("flipped", \
						m_gate, /obj/gate/proc/change_state)

		// Note that we can also connect slots by name!
		// In call(), underscores in proc names become spaces!

				m_gate = locate("gate3_tag")
				if(m_gate)
					signals.connect("flipped", m_gate, "change state")

/******************************************************
	The rest is basic demo stuff, you can ignore it.
******************************************************/

world
	name = "Signals and Slots by Kuraudo"
	turf = /turf/grass
	view = 6

turf
	icon = 'turfs.dmi'

	grass/icon_state = "grass"

	wall
		icon_state = "wall"
		density = 1

mob
	icon = 'mob.dmi'
	New()
		loc = locate(7,2,1)